import argparse
import configparser

from flask import Flask, send_from_directory
from gakeshadeba import Gakeshadeba

parser = argparse.ArgumentParser(
    prog = "Gakeshadeba Server",
    description = "Simple ShaCache and ShaDir implementation in plain Python"
)
parser.add_argument("-c", "--cfg", required=True)
argv = parser.parse_args()

config = configparser.ConfigParser()
config.read(argv.cfg)

gsha = Gakeshadeba(config["gakeshadeba"]["content_directory"], config["gakeshadeba"]["sr_list_path"])
gsha.set_signing_key(config["gakeshadeba"]["signing_key_path"])
gsha.build_file_cache()
gsha.build_sr_dir()

app = Flask(__name__)

@app.route("/bdir/<md5>")
def binary_dir(md5):
    return gsha.serve_sr_dir_entry(md5)

@app.route("/bcache/<sha512>")
def binary_cache(sha512):
    file_name = gsha.find_file_cache_entry(sha512)
    if file_name is None:
        return "Not found\n", 404

    return send_from_directory(config["gakeshadeba"]["content_directory"], file_name, as_attachment=True)

app.run(host=config["server"]["host"], port=config["server"]["port"])

